package com.streambase.contrib.simple;

import java.util.ArrayList;
import java.util.List;
import com.streambase.sb.CompleteDataType;
import com.streambase.sb.DataType;
import com.streambase.sb.client.CustomFunctionResolver;


public class GenerateBM {
			
	/**
	 * A continuous-time stochastic process whose probability distribution is the 
	 * conditional probability distribution of an Wiener process
	 * @param x0 the starting point of the particle at time zero
	 * @param stockT the final point of the particle at the last time  
	 * @param timeT the final time
	 * @param mu the drift of the particle
	 * @param sigma the volatility of the particle
	 * @param multiplier a multiplier so that 2^multiplier data points are generated
	 * @return a vector of 2^multiplier prices 
	 */
	@CustomFunctionResolver("brownianBridgeCustomFunctionResolver")
	public static List<Double> BM_Bridge(Double x0, Double xT, Double timeT, Double mu, Double sigma, Integer multiplier) {
		if (timeT==null || x0 == null || xT == null || multiplier == null || mu == null || sigma == null ||timeT < 0 || sigma <= 0 || multiplier < 1 || multiplier > 23)
			return null;
		double timeRight, timeLeft, timeDenominator, drift, vol;
		int h = (int) Math.pow(2, multiplier);		
		int left, right, iMin, i;
		double deltaT =timeT/h;
		double jMax = 1;
		double[] Z;
		if (!Utilities.seedGenerator())
			return null;
		Z = Utilities.generateNormalsDrift(mu*deltaT, sigma*sigma*deltaT, h);
		double [] brownian = Utilities.generateZeros(h+1);		// generate a vector of zeros
		brownian[0] = x0; brownian[h] = xT;
		double [] intervals = Utilities.generateIntervals(deltaT, h+1);
		for (int k = 0; k < multiplier; k++){
			iMin = (int) h/2; i = iMin;
			left = 0; right = h;	
			for (int j = 0; j < jMax; j++){
				timeRight = intervals[right] - intervals[i];
				timeLeft = intervals[i] - intervals[left];
				timeDenominator = intervals[right] - intervals[left];
				drift = (timeRight * brownian[left] + timeLeft * brownian[right])/timeDenominator;
				vol = sigma*Math.sqrt(timeRight * timeLeft / timeDenominator);
				brownian[i] = drift + vol * Z[i];
				i = i + h; left = left + h; right = right + h;			
			}
			jMax = 2*jMax;
			h = Math.max(iMin, 0);
		}		
		return Utilities.toList(brownian);
	}
			
	/**
	 * Generate a Brownian path
	 * @param x0 the starting point of the particle at time zero
	 * @param stockT the final point of the particle at the last time  
	 * @param timeT the final time
	 * @param mu the drift of the particle
	 * @param sigma the volatility of the particle
	 * @return a vector of points + 1 prices simulated according to brownian motion
	 * @return
	 */
	@CustomFunctionResolver("brownianMotionCustomFunctionResolver")
	public static List<Double> BM(Double x0, Double timeT, Double mu, Double sigma, Integer points) {
		if (x0 == null || points == null || timeT == null || mu ==null || sigma == null  ||timeT < 0 ||  points <= 0 || sigma <= 0)
			return null;
		List<Double> answer = new ArrayList<Double>();
		double deltaT = timeT/points, s0 = x0;
		double[] Z;
		if (!Utilities.seedGenerator())
			return null;
		 Z = Utilities.generateNormalsDrift(mu*deltaT, sigma*sigma*deltaT, points);
		answer.add(s0);
		for (int i = 0; i < points; i++){
			s0 += Z[i];
			answer.add(s0);
		}
		return answer;	 
	}
				
	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	 */
	public static CompleteDataType brownianBridgeCustomFunctionResolver(CompleteDataType x0, CompleteDataType xT, 
			CompleteDataType timeT, CompleteDataType mu, CompleteDataType sigma, CompleteDataType multiplier) {
		boolean a1 = multiplier.getDataType() == DataType.INT && timeT.getDataType() == DataType.DOUBLE;
		boolean a2 = x0.getDataType() == DataType.DOUBLE && xT.getDataType() == DataType.DOUBLE;
		boolean a3 = mu.getDataType() == DataType.DOUBLE && sigma.getDataType() == DataType.DOUBLE; 
		if (a1 && a2 && a3)
			return CompleteDataType.forList(CompleteDataType.forDouble());
		return null;
	}
	
	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	 */
	public static CompleteDataType brownianMotionCustomFunctionResolver(CompleteDataType stock0,  
			CompleteDataType timeT, CompleteDataType mu, CompleteDataType sigma, CompleteDataType multiplier) {
		boolean a1 = multiplier.getDataType() == DataType.INT && timeT.getDataType() == DataType.DOUBLE;
		boolean a2 = mu.getDataType() == DataType.DOUBLE && sigma.getDataType() == DataType.DOUBLE; 
		if (a1 && a2 && stock0.getDataType() == DataType.DOUBLE)
			return CompleteDataType.forList(CompleteDataType.forDouble());
		return null;
	}

}
