package com.streambase.contrib.simple;

import java.util.List;
import com.streambase.sb.CompleteDataType;
import com.streambase.sb.DataType;
import com.streambase.sb.client.CustomFunctionResolver;


public class GenerateGBM {
	
	/**
	 * A continuous-time stochastic process whose probability distribution is the 
	 * conditional probability distribution of an Wiener process
	 * @param stock0 the stock price at time zero
	 * @param stockT the stock price at the final time
	 * @param time0 the initial time
	 * @param timeT the final time
	 * @param multiplier a multiplier so that 2^multiplier prices are generated
	 * @return a vector of 2^multiplier prices 
	 */
	@CustomFunctionResolver("brownianBridgePricesCustomResolver")
	public static List<Double> GBM_Bridge(Double stock0, Double stockT, Double timeT, Double mu, Double sigma, Integer multiplier) {
		return Utilities.exponentiate(GenerateBM.BM_Bridge(Math.log(stock0), Math.log(stockT), timeT, mu , sigma, multiplier));
	}
	
	/**
	 * A continuous-time stochastic process whose probability distribution is the 
	 * conditional probability distribution of an Wiener process
	 * @param stock0 the stock price at time beginning time
	 * @param timeT the final time
	 * @param mu the drift of the asset
	 * @param sigma the volatility of the asset
	 * @param points the number of data points to generate 
	 * @return a vector of points + 1 prices simulated according to exponential brownian motion 
	 */
	@CustomFunctionResolver("brownianMotionPricesCustomResolver")
	public static List<Double> GBM(Double stock0, Double timeT, Double mu, Double sigma, Integer points) {
		return Utilities.exponentiate(GenerateBM.BM(Math.log(stock0), timeT, mu - 0.5*sigma*sigma, sigma, points));
	}
	
	
	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	 */
	public static CompleteDataType brownianBridgePricesCustomResolver(CompleteDataType x0, CompleteDataType xT, 
			CompleteDataType timeT, CompleteDataType mu, CompleteDataType sigma, CompleteDataType multiplier) {
		boolean a1 = multiplier.getDataType() == DataType.INT && timeT.getDataType() == DataType.DOUBLE;
		boolean a2 = x0.getDataType() == DataType.DOUBLE && xT.getDataType() == DataType.DOUBLE;
		boolean a3 = mu.getDataType() == DataType.DOUBLE && sigma.getDataType() == DataType.DOUBLE; 
		if (a1 && a2 && a3)
			return CompleteDataType.forList(CompleteDataType.forDouble());
		return null;
	}
	
	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	 */
	public static CompleteDataType brownianMotionPricesCustomResolver(CompleteDataType stock0,  
			CompleteDataType timeT, CompleteDataType mu, CompleteDataType sigma, CompleteDataType multiplier) {
		boolean a1 = multiplier.getDataType() == DataType.INT && timeT.getDataType() == DataType.DOUBLE;
		boolean a2 = mu.getDataType() == DataType.DOUBLE && sigma.getDataType() == DataType.DOUBLE; 
		if (a1 && a2 && stock0.getDataType() == DataType.DOUBLE)
			return CompleteDataType.forList(CompleteDataType.forDouble());
		return null;
	}

}
