package com.streambase.contrib.simple;

import org.uncommons.maths.random.ContinuousUniformGenerator;
import org.uncommons.maths.random.GaussianGenerator;
import org.uncommons.maths.random.MersenneTwisterRNG;
import org.uncommons.maths.random.SecureRandomSeedGenerator;
import org.uncommons.maths.random.SeedException;
import com.streambase.sb.CompleteDataType;
import com.streambase.sb.DataType;
import com.streambase.sb.NullValueException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.client.CustomFunctionResolver;


public class Triplets {
	
	private static GaussianGenerator gaussian = null;
	private static ContinuousUniformGenerator uniform = null;	
		
	/**
	 * Computes a triple for Brownian motion
	 * @param T the final time
	 * @param stock0 the stock price at time beginning time
	 * @param mu the drift of the asset
	 * @param sigma the volatility of the asset
	 * @return a triplet representing the final, min and max realization of the BM
	 */
	@CustomFunctionResolver("tripletCustomFunctionResolver")
	public static Tuple GBM_Triplet(Double stock0, Double mu, Double sigma, Double T){
		if ( T == null || sigma == null || mu == null || stock0 == null || stock0 == 0 || T < 0 || sigma < 0 || !seedGenerator())				
				return null;	
		double z = gaussian.nextValue() + mu*Math.sqrt(T)/sigma;
		double a = 0.5*z - Math.sqrt(0.25*z - 0.5*Math.log(uniform.nextValue()));					
		Tuple out = ArrivalType.SCHEMA.createTuple();		
		try {
			out.setDouble("Final", stock0*Math.exp(Math.sqrt(T)*sigma*z));
			out.setDouble("Minimum", stock0*Math.exp(Math.sqrt(T)*sigma*a));
			out.setDouble("Maximum", stock0*Math.exp(Math.sqrt(T)*sigma*Utilities.FindMax(z,a, uniform.nextValue())));
		} catch (NullValueException e) {
			System.out.println("NullValueException: Tuple arguments are Null.");
			return null;
		} catch (TupleException e) {
			System.out.println("TupleException: Cannot set create new Tuple.");
			return null;
		}
		return out;
	}	
	
	/**
	 * Computes a triple for Brownian motion
	 * @param T the final time
	 * @param x0 the value of the BM at the beginning of time
	 * @param mu the drift of the asset
	 * @param sigma the volatility of the asset
	 * @return a triplet representing the final, min and max realization of the BM
	 */
	@CustomFunctionResolver("tripletCustomFunctionResolver")
	public static Tuple BM_Triplet(Double x0, Double mu, Double sigma, Double T){
		if (T == null || sigma == null || mu == null || x0 == null || T < 0 || sigma < 0 || !seedGenerator())				
				return null;	
		double z = gaussian.nextValue() + mu*Math.sqrt(T)/sigma;
		double a = 0.5*z - Math.sqrt(0.25*z - 0.5*Math.log(uniform.nextValue()));					
		Tuple out = ArrivalType.SCHEMA.createTuple();		
		try {
			out.setDouble("Final", x0 + Math.sqrt(T)*sigma*z);
			out.setDouble("Minimum", x0 + Math.sqrt(T)*sigma*a);
			out.setDouble("Maximum", x0 + Math.sqrt(T)*sigma*Utilities.FindMax(z,a, uniform.nextValue()));
		} catch (NullValueException e) {
			System.out.println("NullValueException: Tuple arguments are Null.");
			return null;
		} catch (TupleException e) {
			System.out.println("TupleException: Cannot set create new Tuple.");
			return null;
		}
		return out;
	}	

	/**
	 * Seeds the generator only once if it hasn't been seeded 
	 * @return true if generator is instantiated, false otherwise
	 */
	private static boolean seedGenerator(){
		if (gaussian == null || uniform == null){
			try {
				gaussian = new GaussianGenerator(0,1, new MersenneTwisterRNG(new SecureRandomSeedGenerator().generateSeed(Utilities.MERSENNE_BYTES)));
				uniform = new ContinuousUniformGenerator(0,1, new MersenneTwisterRNG(new SecureRandomSeedGenerator().generateSeed(Utilities.MERSENNE_BYTES)));
			} catch (SeedException e) {
				System.out.println("SeedException: Cannot seed RNG.");
				return false;
			}
		}
		return true;		
	}
	
	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	 */
	public static CompleteDataType tripletCustomFunctionResolver(CompleteDataType stock0, CompleteDataType mu, CompleteDataType sigma, CompleteDataType T) {
		boolean a1 = T.getDataType() == DataType.DOUBLE && stock0.getDataType() == DataType.DOUBLE;
		boolean a2 = mu.getDataType() == DataType.DOUBLE && sigma.getDataType() == DataType.DOUBLE; 
		if (a1 && a2)
			return CompleteDataType.forTuple(ArrivalType.SCHEMA);
		return null;
	}

}
