package com.streambase.contrib.simple;

import java.util.ArrayList;
import java.util.List;

import org.uncommons.maths.random.GaussianGenerator;
import org.uncommons.maths.random.MersenneTwisterRNG;
import org.uncommons.maths.random.SecureRandomSeedGenerator;
import org.uncommons.maths.random.SeedException;


public class Utilities {
	
	private static GaussianGenerator generator = null;
	public static final double EPSILON = Math.pow(10, -8);
	public static final double DELTA = Math.pow(10, -12);
	public static final int MERSENNE_BYTES = 16;
					
	/**
	 * 
	 * @param z	the realization of regular BM
	 * @param a the minimum value of the BM over the time period
	 * @param u a uniform random variate (0,1]
	 * @return the maximum value of the BM over the time period
	 */
	public static double FindMax(double z, double a, double u){
		double l = Math.max(z,0), r = 1.0 / 0.0, f, F, x0;  
		double xn = 0.5*(z + Math.max(Math.log(1.0/(Math.max(z, 0) - a)), 0)) + Math.sqrt(0.25*z*z - 0.5*(1 + Math.log (1 + a*a/(Math.abs(z) + 0.01)))*Math.log (1.0 - u));
		do{
			x0 = xn;
			f = PDF(z,a,x0);
			F = CDF(z,a,x0);
			if (F < u) {l = x0;} else {r = x0;}
			xn = x0 - (F - u)/f;
			if (xn < l || xn > r){
				xn = (r != 1.0/0.0) ? 0.5*(l+r) : 1.2*(l+0.1);
			}
		} while((xn - x0) >= EPSILON);		
		return 0.5*(xn + x0);
	}

	/**
	 * 
	 * @param z the realization of regular BM
	 * @param a the minimum value of the BM over the time period
	 * @param b the maximum value of the BM over the time period
	 * @return  an approximation to the CDF of F(B|Z=z, A=a)
	 */
	public static double CDF(double z, double a, double b){
		double sum = Math.exp(2*a*(z-a))*(z-2*a);
		double termPos = 1.0/0.0, termNeg = 1.0/0.0, squared = 0.5*z*z; 
		double tempPos1, tempNeg1, tempPos2, tempNeg2;
		int k = 0;
		do {
			k++;
			tempPos1 = z-2*a+2*k*(b-a); tempNeg1 = z-2*a-2*k*(b-a); tempPos2 = tempPos1 + 2*a; tempNeg2 = tempNeg1 + 2*a;
			termPos = (k+1)*tempPos1*Math.exp(-0.5*tempPos1*tempPos1 + squared) - k*tempPos2*Math.exp(-0.5*tempPos2*tempPos2 + squared); 
			termNeg = (1-k)*tempNeg1*Math.exp(-0.5*tempNeg1*tempNeg1 + squared) + k*tempNeg2*Math.exp(-0.5*tempNeg2*tempNeg2 + squared);
			sum += termPos + termNeg;	
		} while(termPos > DELTA && termNeg > DELTA);
		return Math.exp(2*a*(a-z))/(z-2*a)*sum;
	}
	
	/**
	 * 
	 * @param z the realization of regular BM
	 * @param a the minimum value of the BM over the time period
	 * @param b the maximum value of the BM over the time period
	 * @return  an approximation to the CDF of f(B|Z=z, A=a)
	 */
	public static double PDF(double z, double a, double b){
		double sum = 0, squared = 0.5*z*z;
		double termPos = 1.0/0.0, termNeg = 1.0/0.0; 
		double tempPos1, tempNeg1, tempPos2, tempNeg2; 
		int k = 0;
		do {
			k++;
			tempPos1 = z-2*a+2*k*(b-a); tempNeg1 = z-2*a-2*k*(b-a); tempPos2 = tempPos1 + 2*a; tempNeg2 = tempNeg1 + 2*a;
			termPos = 2*k*(k+1)*(1-tempPos1*tempPos1)*Math.exp(-0.5*tempPos1*tempPos1 + squared) - 2*k*k*(1-tempPos2*tempPos2)*Math.exp(-0.5*tempPos2*tempPos2 + squared); 
			termNeg =-2*k*(1-k)*(1-tempNeg1*tempNeg1)*Math.exp(-0.5*tempNeg1*tempNeg1 + squared) - 2*k*k*(1-tempNeg2*tempNeg2)*Math.exp(-0.5*tempNeg2*tempNeg2 + squared);
			sum += termPos + termNeg;	
		} while(termPos > DELTA && termNeg > DELTA);
		return Math.exp(2*a*(a-z))/(z-2*a)*sum;
	}
			
	/**
	 * Generates a vector containing n zeros 
	 * @param n the length of the vector
	 * @return a double array of zeros
	 */
	public static double [] generateZeros(int n){
		double [] zeros = new double[n];
		for (int i = 0; i < n; i++){
			zeros[i]= 0.0;
		}
		return zeros;
	}
	
	/**
	 * Generates a vector containing n times starting at time0
	 * @param time0 the initial time to start the vector
	 * @param deltaT the time increment 
	 * @param n the number of timestamps
	 * @return a vector a n times
	 */
	public static double [] generateIntervals(double deltaT, int n){
		double [] times = new double[n];
		times[0] = 0;
		for (int i = 1; i < n; i++){
			times[i] = times[i-1] + deltaT;
		}
		return times;
	}
	
	/**
	 * Generates a list of exponentiated prices
	 * @param brownian an array of brownian motions
	 * @return a list of exponentiated prices
	 */
	public static List<Double> exponentiate(List<Double> brownian){
		List<Double> output = new ArrayList<Double>();	
		for (Double brown : brownian){
			output.add(Math.exp(brown));
		}
		return output;
	}
	
	/**
	 * Generates a list of exponentiated prices
	 * @param brownian an array of brownian motions
	 * @return a list of exponentiated prices
	 */
	public static List<Double> toList(double [] brownian){
		List<Double> output = new ArrayList<Double>();	
		for (double brown : brownian){
			output.add(brown);
		}
		return output;
	}
	
	
	/**
	 * Generates n standard normal random variables.
	 * @param n the number of random variates to generate
	 * @param deltaT the time interval between prices
	 * @return a double array of standard normal random variates
	 * @throws SeedException 
	 */
	public static double [] generateNormals(double deltaT, int n){
		 double [] stdNormals = new double[n];
		 for (int i = 0; i < n; i++){
			 stdNormals[i] = Math.sqrt(deltaT)*generator.nextValue();
		 }
		return stdNormals;
	}
	
	/**
	 * Generates n standard normal random variables.
	 * @param n the number of random variates to generate
	 * @param drift the drift of the weiner process
	 * @param deltaT the time interval between prices
	 * @return a double array of standard normal random variates
	 * @throws SeedException 
	 */
	public static double [] generateNormalsDrift(double drift, double deltaT, int n){	
		 double [] stdNormals = new double[n];
		 for (int i = 0; i < n; i++){
			 stdNormals[i] = drift + Math.sqrt(deltaT)*generator.nextValue();
		 }
		return stdNormals;
	}
		
	/**
	 * Seeds the generator only once if it hasn't been seeded 
	 * @return true if generator is instantiated, false otherwise
	 */
	public static boolean seedGenerator(){
		if (generator == null){
			try {
				generator = new GaussianGenerator(0,1, new MersenneTwisterRNG(new SecureRandomSeedGenerator().generateSeed(MERSENNE_BYTES)));
			} catch (SeedException e) {
				System.out.println("SeedException: Cannot seed RNG.");
				return false;
			}
		}
		return true;		
	}
}
